package net.java.dev.annsor;

/**
 * @author Richard L. Burton III
 * @version 1.0
 */
class StatefulProcessorFactoryImpl implements ProcessorFactory {

    private ProcessorFactory factory;

    private String factoryLocationKey;


    StatefulProcessorFactoryImpl() {
    }

    public StatefulProcessorFactoryImpl(ProcessorFactory factory) {
        this.factory = factory;
        factoryLocationKey = factory.getClass().getName();
    }

    /**
     * This method is invoked when a registered annotation is found.
     * @param annotationElement The AnnotationElement used for consuming
     *                          the registered annotation.
     * @return A new Processor that either implements Processor or is
     *         decorated with Annsor annotations.
     * @throws Exception Thrown if there is a problem with a subsystem.
     */
    public Object create(AnnotationElement annotationElement) throws Exception {
        DefaultAnnotationElement element = (DefaultAnnotationElement) annotationElement;
        Object processor = element.getStatefulProcessor(factoryLocationKey);
        if (processor == null) {
            processor = factory.create(annotationElement);
            element.addStatefulProcessor(factoryLocationKey, processor);
            element.addStatefulProcessorFactory(factoryLocationKey, factory);
        }
        return processor;
    }

}
