package net.java.dev.annsor.descriptor;

import java.lang.annotation.Annotation;
import java.lang.reflect.AnnotatedElement;
import java.util.Arrays;

/**
 * TODO: Handle exposing annotations without having to clone the array.
 * @author Richard L. Burton III
 */
public class AttributeDescriptor<T extends AnnotatedElement> implements Descriptor {

    /**
     * The Annotations associated with this constructor.
     */
    private final Annotation[] annotations;

    /**
     * The attributes this descriptor represents.
     */
    private final T attribute;

    public AttributeDescriptor(T attribute) {
        this.attribute = attribute;
        this.annotations = attribute.getDeclaredAnnotations();
    }

    public AttributeDescriptor(T attribute, Annotation... annotation) {
        this.annotations = annotation;
        this.attribute = attribute;
    }

    public Annotation[] getAnnotations() {
        return annotations;
    }

    protected void addAnnotations(Annotation[] annotations){
        System.arraycopy(this.annotations, 0, annotations, 0, annotations.length);
    }

    public T getAttribute() {
        return attribute;
    }

    public boolean hasAnnotations() {
        return annotations.length != 0;
    }

    public String toString() {
        return "AttributeDescriptor{" +
                "annotations=" + (annotations == null ? null : Arrays.asList(annotations)) +
                ", attribute=" + attribute +
                '}';
    }

}
