package net.java.dev.annsor.processors;

import net.java.dev.annsor.Scope;
import net.java.dev.annsor.annotations.Create;
import net.java.dev.annsor.annotations.Destroy;
import net.java.dev.annsor.annotations.Processor;

/**
 * During some performance testing, the invoking methods on the annotations
 * consumed 300+ms for 1,000,000 iterations. In order to improve performance,
 * the values of the @Procesor and @Create annotations are cached. Really, is
 * this worth the 300+ms gained :) I doubt!
 * @author Richard L. Burton III
 * @version 1.0
 */
class PojoMethodParameters {

    private int targetObjectIndex = -1;

    private int annotationIndex = -1;

    private int methodIndex = -1;

    private int fieldIndex = -1;

    private int fieldNameIndex = -1;

    private int annotationTypeIndex = -1;

    private int methodNameIndex = -1;

    private int targetClassIndex = -1;

    private int targetClassNameIndex = -1;

    private int methodParametersIndex = -1;

    private int annsorContextTypeIndex = -1;

    private Scope[] scope;

    public PojoMethodParameters(Destroy processor) {
        targetObjectIndex = processor.targetObjectIndex();
        annotationIndex = processor.annotationIndex();
        methodIndex = processor.methodIndex();
        fieldIndex = processor.fieldIndex();
        fieldNameIndex = processor.fieldNameIndex();
        annotationTypeIndex = processor.annotationTypeIndex();
        methodNameIndex = processor.methodNameIndex();
        targetClassIndex = processor.targetClassIndex();
        targetClassNameIndex = processor.targetClassNameIndex();
        methodParametersIndex = processor.methodParametersIndex();
        annsorContextTypeIndex = processor.annsorContextIndex();
    }

    public PojoMethodParameters(Processor processor) {
        targetObjectIndex = processor.targetObjectIndex();
        annotationIndex = processor.annotationIndex();
        methodIndex = processor.methodIndex();
        fieldIndex = processor.fieldIndex();
        fieldNameIndex = processor.fieldNameIndex();
        annotationTypeIndex = processor.annotationTypeIndex();
        methodNameIndex = processor.methodNameIndex();
        targetClassIndex = processor.targetClassIndex();
        targetClassNameIndex = processor.targetClassNameIndex();
        methodParametersIndex = processor.methodParametersIndex();
        annsorContextTypeIndex = processor.annsorContextIndex();
        scope = processor.scope();
    }

    public PojoMethodParameters(Create processor) {
        targetObjectIndex = processor.targetObjectIndex();
        annotationIndex = processor.annotationIndex();
        methodIndex = processor.methodIndex();
        fieldIndex = processor.fieldIndex();
        fieldNameIndex = processor.fieldNameIndex();
        annotationTypeIndex = processor.annotationTypeIndex();
        methodNameIndex = processor.methodNameIndex();
        targetClassIndex = processor.targetClassIndex();
        targetClassNameIndex = processor.targetClassNameIndex();
        methodParametersIndex = processor.methodParametersIndex();
        annsorContextTypeIndex = processor.annsorContextIndex();
        scope = processor.scope();

    }

    public Scope[] getScope() {
        return scope;
    }

    public int getAnnotationIndex() {
        return annotationIndex;
    }

    public int getAnnotationTypeIndex() {
        return annotationTypeIndex;
    }

    public int getFieldIndex() {
        return fieldIndex;
    }

    public int getFieldNameIndex() {
        return fieldNameIndex;
    }

    public int getMethodIndex() {
        return methodIndex;
    }

    public int getMethodNameIndex() {
        return methodNameIndex;
    }

    public int getTargetClassIndex() {
        return targetClassIndex;
    }

    public int getTargetClassNameIndex() {
        return targetClassNameIndex;
    }

    public int getTargetObjectIndex() {
        return targetObjectIndex;
    }

    public int getMethodParametersIndex() {
        return methodParametersIndex;
    }

    public int getAnnsorContextIndex(){
        return annsorContextTypeIndex;
    }

    /**
     * Returns true if the targetObjectIndex was set on the Processor annotation.
     * @return true if set, false otherwise.
     */
    boolean isTargetObjectIndex() {
        return targetObjectIndex != -1;
    }

    /**
     * Returns true if the methodIndex was set on the Processor annotation.
     * @return true if set, false otherwise.
     */
    boolean isMethodIndex() {
        return methodIndex != -1;
    }

    /**
     * Returns true if the methodIndex was set on the Processor annotation.
     * @return true if set, false otherwise.
     */
    boolean isMethodParametersIndex() {
        return methodParametersIndex != -1;
    }

    /**
     * Returns true if the fieldIndex was set on the Processor annotation.
     * @return true if set, false otherwise.
     */
    boolean isFieldIndex() {
        return fieldIndex != -1;
    }

    /**
     * Returns true if the annotationIndex was set on the Processor annotation.
     * @return true if set, false otherwise.
     */
    boolean isAnnotationIndex() {
        return annotationIndex != -1;
    }

    /**
     * Returns true if the methodNameIndex was set on the Processor annotation.
     * @return true if set, false otherwise.
     */
    boolean isMethodNameIndex() {
        return annotationIndex != -1;
    }

    /**
     * Returns true if the fieldNameIndex was set on the Processor annotation.
     * @return true if set, false otherwise.
     */
    boolean isFieldNameIndex() {
        return annotationIndex != -1;
    }

    /**
     * Returns true if the targetClassIndex was set on the Processor annotation.
     * @return true if set, false otherwise.
     */
    boolean isTargetClassIndex() {
        return annotationIndex != -1;
    }

    /**
     * Returns true if the targetClassNameIndex was set on the Processor annotation.
     * @return true if set, false otherwise.
     */
    boolean isTargetClassNameIndex() {
        return annotationIndex != -1;
    }

    /**
     * Returns true if the annotationTypeIndex was set on the Processor annotation.
     * @return true if set, false otherwise.
     */
    boolean isAnnotationTypeIndex() {
        return annotationTypeIndex != -1;
    }

    public boolean isAnnsorContext() {
        return annsorContextTypeIndex != -1;
    }
}
