package net.java.dev.annsor.processors;

import net.java.dev.annsor.AnnotationElement;
import net.java.dev.annsor.StatefulProcessorFactory;
import net.java.dev.annsor.annotations.Destroy;

import java.lang.reflect.Method;

/**
 * @author Richard L. Burton III
 * @version 1.0
 */
public class StatefulPojoProcessorFactory extends PojoProcessorFactory implements StatefulProcessorFactory {

    /**
     * The <tt>create</tt> method to be invoked in order to create a new
     * custom Processor object.
     */
    protected PojoMethod destroyMethod;

    /**
     * Constructs a new PojoProcessorFactory from a Pojo that was decorated with the @Factory
     * and @Create annotations.
     * @param target The object that will be used for the Factory.
     * @throws IllegalArgumentException Thrown if there was no @Create annotation on the
     *                                  factory object.
     */
    public StatefulPojoProcessorFactory(Object target) {
        super(target);
    }

    public void destroy(AnnotationElement annotationElement) throws Exception {
        if (destroyMethod != null) {
            destroyMethod.invoke(annotationElement, target);
        }
    }

    @Override
    protected void inspectMethod(Method method, Object target) {
        super.inspectMethod(method, target);
        if (method.isAnnotationPresent(Destroy.class)) {
            destroyMethod = new PojoMethod(method.getAnnotation(Destroy.class), target, method);
        }
    }
}
